@media (prefers-color-scheme: dark) {
  :root {
    background-color: #333;
    color: #eee;
  }

  footer {
    background-color: var(--bg-black);
    color: #eee;
    border: none;
  }

  .follows--title.non-empty,
  .title--descriptions {
    background-color: #555;
  }

  a.touch-friendly {
    background-color: silver;
  }
}
